unit MRDBEdit;

{*********************************************************
 *****                                               *****
 *****                 TMRDBEDIT                     *****
 *****                                               *****
 *********************************************************

 -Written By Martin Rodemann
 -Last Modified on 25.October 1998

 -MRDBEdit is a modified TEdit97 from the famous TToolbar97
  components-pack
 -Version History: (+ New Function * Bugfixes)

  -V1.03 First Release (what you see now)
   +ET_MREDITStyle has been created for Editstyle-Property
   *Changing of Editstyleproperty while Programm running
    hasn't updated actual Display
  -V1.02
   +Editstyle
  -V1.01
   +Changed to "3D-Look and Feel" Drawing
  -V1.00
   Created TMRDBEdit for use of Toolbar97 like Style for
   TDBEdit

  Thanx to Jordan Russell for his famous Toolbar97 components
  (I will register soon)

  This component is Freeware.
  Please send eMail to :martin.rodemann@essen.netsurf.de
  if you are using this component or for sugestions
  Please also send me any changing of code, I'm still learning
  (like all programmers !)

  This is my first component-release !

  }

interface

uses
  Windows, Messages,SysUtils, Classes, Controls, Forms, Menus, Graphics, Buttons,
  StdCtrls, ExtCtrls,Mask,DbCtrls,DB;

type
  TMREditstyle = (ET_MREDITStyle,ET_OLDTOOLBAR97Style,ET_NEWTOOLBAR97Style);

type
  TMRDBEdit = class(TDBEdit)
  private
    FEdittype : TMREditstyle;
    MouseInControl: Boolean;
    procedure RedrawBorder (const Clip: HRGN);
    procedure NewAdjustHeight;
    procedure KeyDown(var Key: Word; Shift: TShiftState);       override;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMSetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus (var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMNCCalcSize (var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint (var Message: TMessage); message WM_NCPAINT;
    Procedure SetEdittype (value : TMREditstyle);
  protected
    procedure Loaded; override;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property CharCase;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Editstyle : TMREditstyle Read FEdittype Write SetEdittype;
    {$IFDEF TB97Delphi3orHigher}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure TMRDBEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
 inherited KeyDown(Key, Shift);
    if ((Key=VK_DOWN)) or
       ((Key=VK_RETURN)) then
      (Owner as TForm).Perform(WM_NEXTDLGCTL,0,0)
    else if (Key=VK_UP) then
      (Owner as TForm).Perform(WM_NEXTDLGCTL,1,0);
end;

constructor TMRDBEdit.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  AutoSize := False;
  Ctl3D := False;
  BorderStyle := bsNone;
  ControlStyle := ControlStyle - [csFramed]; {fixes a VCL bug with Win 3.x}
  Height := 19;
end;

procedure TMRDBEdit.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  MouseInControl := True;
  RedrawBorder (0);
end;

procedure TMRDBEdit.CMMouseLeave (var Message: TMessage);
begin
  inherited;
  MouseInControl := False;
  RedrawBorder (0);
end;

procedure TMRDBEdit.NewAdjustHeight;
var
  DC: HDC;
  SaveFont: HFONT;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics (DC, Metrics);
  SelectObject (DC, SaveFont);
  ReleaseDC (0, DC);

  Height := Metrics.tmHeight + 6;
end;

procedure TMRDBEdit.Loaded;
begin
  inherited Loaded;
  if not(csDesigning in ComponentState) then
    NewAdjustHeight;
end;

procedure TMRDBEdit.CMEnabledChanged (var Message: TMessage);
const
  EnableColors: array[Boolean] of TColor = (clBtnFace, clWindow);
begin
  inherited;
  Color := EnableColors[Enabled];
end;

procedure TMRDBEdit.CMFontChanged (var Message: TMessage);
begin
  inherited;
  if not((csDesigning in ComponentState) and (csLoading in ComponentState)) then
    NewAdjustHeight;
end;

procedure TMRDBEdit.WMSetFocus (var Message: TWMSetFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    RedrawBorder (0);
  IF Field<>nil then
   begin
    IF Field.DataType=ftstring then
    Maxlength:=Field.DisplayWidth;
   end;
end;

procedure TMRDBEdit.WMKillFocus (var Message: TWMKillFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    RedrawBorder (0);
end;

procedure TMRDBEdit.WMNCCalcSize (var Message: TWMNCCalcSize);
begin
  inherited;
  InflateRect (Message.CalcSize_Params^.rgrc[0], -3, -3);
end;


procedure TMRDBEdit.WMNCPaint (var Message: TMessage);
begin
  inherited;
  RedrawBorder (Message.WParam);
end;

procedure TMRDBEdit.RedrawBorder (const Clip: HRGN);
var
  DC: HDC;
  R: TRect;
  NewClipRgn: HRGN;
  BtnFaceBrush, WindowBrush: HBRUSH;
  FCanvas : TCanvas;
begin
  FCanvas:=TCanvas.Create;
  DC := GetWindowDC(Handle);
  try
    if (Clip <> 0) and (Clip <> 1) then begin
      GetWindowRect (Handle, R);
      if SelectClipRgn(DC, Clip) = ERROR then begin
        NewClipRgn := CreateRectRgnIndirect(R);
        SelectClipRgn (DC, NewClipRgn);
        DeleteObject (NewClipRgn);
      end;
      OffsetClipRgn (DC, -R.Left, -R.Top);
    end;
    GetWindowRect (Handle, R);  OffsetRect (R, -R.Left, -R.Top);
    BtnFaceBrush := GetSysColorBrush(COLOR_BTNFACE);
    WindowBrush := GetSysColorBrush(COLOR_WINDOW);
    if ((csDesigning in ComponentState) and Enabled) or
       (not(csDesigning in ComponentState) and
        (Focused or (MouseInControl and not(Screen.ActiveControl is TMRDBEdit)))) then begin
      DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      with R do begin
        FillRect (DC, Rect(Left, Top, Left+1, Bottom-1), BtnFaceBrush);
        FillRect (DC, Rect(Left, Top, Right-1, Top+1), BtnFaceBrush);
      end;
      DrawEdge (DC, R, BDR_SUNKENINNER, BF_BOTTOMRIGHT);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
      If FEdittype=ET_MREDITStyle then
      begin
       FCanvas.handle:=DC;
       FCanvas.Pen.Color := clWindowText;
       FCanvas.MoveTo(R.Right, R.Top-1);
       FCanvas.LineTo(R.Right, R.Bottom);
       FCanvas.LineTo(R.Left-3, R.Bottom);
       FCanvas.Pen.Color := clBtnShadow;
       FCanvas.MoveTo(R.Right-1, R.Top);
       FCanvas.LineTo(R.Right-1, R.Bottom-1);
       FCanvas.LineTo(R.Left-2, R.Bottom-1);
      end;
   end
    else begin
     If FEdittype=ET_MREDITStyle then
     begin
      DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      with R do begin
        FillRect (DC, Rect(Left, Top, Left+1, Bottom-1), BtnFaceBrush);
        FillRect (DC, Rect(Left, Top, Right-1, Top+1), BtnFaceBrush);
      end;
      DrawEdge (DC, R, BDR_SUNKENINNER, BF_BOTTOMRIGHT);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
     end
     else
     IF FEdittype=ET_NEWTOOLBAR97Style then
     begin
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, BtnFaceBrush);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, WindowBrush);
     end;
    end;
  finally
    ReleaseDC (Handle, DC);
  end;
FCanvas.Free;
end;

Procedure TMRDBEdit.SetEdittype (value : TMREditstyle);
begin
 IF Value=FEdittype then exit;
 FEdittype:=Value;
 Redrawborder(0);
end;


procedure Register;
begin
  RegisterComponents('MR....', [TMRDBEdit]);
end;

end.
